import { DocumentCapabilities, EmbeddedFileSourceMap, SourceFile, PositionCapabilities, Teleport, TeleportMappingData, EmbeddedFile } from '@volar/language-core';
import { SFCParseResult } from '@vue/compiler-sfc';
import { ComputedRef, Ref } from '@vue/reactivity';
import { Sfc, VueLanguagePlugin } from './types';
import { CodeGen } from '@volar/code-gen';
import { Mapping } from '@volar/source-map';
import * as CompilerDom from '@vue/compiler-dom';
import type * as ts from 'typescript/lib/tsserverlibrary';
export interface EmbeddedStructure {
    self: Embedded | undefined;
    embeddeds: EmbeddedStructure[];
}
export interface Embedded {
    file: VueEmbeddedFile;
    sourceMap: EmbeddedFileSourceMap;
    teleport: Teleport | undefined;
}
export interface VueEmbeddedFile {
    parentFileName?: string;
    fileName: string;
    isTsHostFile: boolean;
    capabilities: DocumentCapabilities;
    codeGen: CodeGen<EmbeddedFileMappingData>;
    teleportMappings: Mapping<TeleportMappingData>[];
}
export interface EmbeddedFileMappingData {
    vueTag: 'template' | 'script' | 'scriptSetup' | 'scriptSrc' | 'style' | 'customBlock' | undefined;
    vueTagIndex?: number;
    capabilities: PositionCapabilities;
}
export declare class VueSourceFile implements SourceFile {
    fileName: string;
    private pscriptSnapshot;
    private ts;
    private plugins;
    sfc: Sfc;
    get text(): string;
    get compiledSFCTemplate(): {
        errors: CompilerDom.CompilerError[];
        warnings: CompilerDom.CompilerError[];
        ast: CompilerDom.RootNode | undefined;
    } | undefined;
    get tsFileName(): string;
    get embeddeds(): EmbeddedFile[];
    _snapshot: Ref<ts.IScriptSnapshot>;
    _text: ComputedRef<string>;
    _parsedSfcCache: {
        snapshot: ts.IScriptSnapshot;
        sfc: SFCParseResult;
        plugin: ReturnType<VueLanguagePlugin>;
    } | undefined;
    _compiledSFCTemplateCache: {
        snapshot: ts.IScriptSnapshot;
        result: CompilerDom.CodegenResult;
        plugin: ReturnType<VueLanguagePlugin>;
    } | undefined;
    _parsedSfc: ComputedRef<SFCParseResult | undefined>;
    _compiledSFCTemplate: ComputedRef<{
        errors: CompilerDom.CompilerError[];
        warnings: CompilerDom.CompilerError[];
        ast: CompilerDom.RootNode | undefined;
    } | undefined>;
    _pluginEmbeddedFiles: ComputedRef<[VueEmbeddedFile, EmbeddedFile][]>[];
    _allEmbeddeds: ComputedRef<[VueEmbeddedFile, EmbeddedFile][]>;
    _embeddeds: ComputedRef<EmbeddedFile[]>;
    constructor(fileName: string, pscriptSnapshot: ts.IScriptSnapshot, ts: typeof import('typescript/lib/tsserverlibrary'), plugins: ReturnType<VueLanguagePlugin>[]);
    update(newScriptSnapshot: ts.IScriptSnapshot, init?: boolean): void;
}
