Object.defineProperty(exports, "__esModule", { value: true });
exports.VueSourceFile = void 0;
const reactivity_1 = require("@vue/reactivity");
const code_gen_1 = require("@volar/code-gen");
;
class VueSourceFile {
    constructor(fileName, pscriptSnapshot, ts, plugins) {
        this.fileName = fileName;
        this.pscriptSnapshot = pscriptSnapshot;
        this.ts = ts;
        this.plugins = plugins;
        this.sfc = (0, reactivity_1.reactive)({
            template: null,
            script: null,
            scriptSetup: null,
            styles: [],
            customBlocks: [],
            templateAst: (0, reactivity_1.computed)(() => {
                var _a;
                return (_a = this._compiledSFCTemplate.value) === null || _a === void 0 ? void 0 : _a.ast;
            }),
            scriptAst: (0, reactivity_1.computed)(() => {
                if (this.sfc.script) {
                    return this.ts.createSourceFile(this.fileName + '.' + this.sfc.script.lang, this.sfc.script.content, this.ts.ScriptTarget.Latest);
                }
            }),
            scriptSetupAst: (0, reactivity_1.computed)(() => {
                if (this.sfc.scriptSetup) {
                    return this.ts.createSourceFile(this.fileName + '.' + this.sfc.scriptSetup.lang, this.sfc.scriptSetup.content, this.ts.ScriptTarget.Latest);
                }
            }),
        }) /* avoid Sfc unwrap in .d.ts by reactive */;
        this._text = (0, reactivity_1.computed)(() => this._snapshot.value.getText(0, this._snapshot.value.getLength()));
        // computeds
        this._parsedSfc = (0, reactivity_1.computed)(() => {
            var _a, _b;
            // incremental update
            if ((_a = this._parsedSfcCache) === null || _a === void 0 ? void 0 : _a.plugin.updateSFC) {
                const change = this._snapshot.value.getChangeRange(this._parsedSfcCache.snapshot);
                if (change) {
                    const newSfc = this._parsedSfcCache.plugin.updateSFC(this._parsedSfcCache.sfc, {
                        start: change.span.start,
                        end: change.span.start + change.span.length,
                        newText: this._snapshot.value.getText(change.span.start, change.span.start + change.newLength),
                    });
                    if (newSfc) {
                        this._parsedSfcCache.snapshot = this._snapshot.value;
                        this._parsedSfcCache.sfc = newSfc;
                        return newSfc;
                    }
                }
            }
            for (const plugin of this.plugins) {
                const sfc = (_b = plugin.parseSFC) === null || _b === void 0 ? void 0 : _b.call(plugin, this.fileName, this._text.value);
                if (sfc) {
                    if (!sfc.errors.length) {
                        this._parsedSfcCache = {
                            snapshot: this._snapshot.value,
                            sfc,
                            plugin,
                        };
                    }
                    return sfc;
                }
            }
        });
        this._compiledSFCTemplate = (0, reactivity_1.computed)(() => {
            var _a, _b;
            if (this.sfc.template) {
                (0, reactivity_1.pauseTracking)();
                // don't tracking
                const newSnapshot = this._snapshot.value;
                const templateOffset = this.sfc.template.startTagEnd;
                (0, reactivity_1.resetTracking)();
                // tracking
                this.sfc.template.content;
                // incremental update
                if ((_a = this._compiledSFCTemplateCache) === null || _a === void 0 ? void 0 : _a.plugin.updateSFCTemplate) {
                    const change = newSnapshot.getChangeRange(this._compiledSFCTemplateCache.snapshot);
                    if (change) {
                        const newText = newSnapshot.getText(change.span.start, change.span.start + change.newLength);
                        const newResult = this._compiledSFCTemplateCache.plugin.updateSFCTemplate(this._compiledSFCTemplateCache.result, {
                            start: change.span.start - templateOffset,
                            end: change.span.start + change.span.length - templateOffset,
                            newText,
                        });
                        if (newResult) {
                            this._compiledSFCTemplateCache.snapshot = newSnapshot;
                            this._compiledSFCTemplateCache.result = newResult;
                            return {
                                errors: [],
                                warnings: [],
                                ast: newResult.ast,
                            };
                        }
                    }
                }
                for (const plugin of this.plugins) {
                    const errors = [];
                    const warnings = [];
                    let result;
                    try {
                        result = (_b = plugin.compileSFCTemplate) === null || _b === void 0 ? void 0 : _b.call(plugin, this.sfc.template.lang, this.sfc.template.content, {
                            onError: (err) => errors.push(err),
                            onWarn: (err) => warnings.push(err),
                            expressionPlugins: ['typescript'],
                        });
                    }
                    catch (e) {
                        const err = e;
                        errors.push(err);
                    }
                    if (result || errors.length) {
                        if (result && !errors.length && !warnings.length) {
                            this._compiledSFCTemplateCache = {
                                snapshot: newSnapshot,
                                result: result,
                                plugin,
                            };
                        }
                        return {
                            errors,
                            warnings,
                            ast: result === null || result === void 0 ? void 0 : result.ast,
                        };
                    }
                }
            }
        });
        this._pluginEmbeddedFiles = this.plugins.map(plugin => {
            const embeddedFiles = {};
            const files = (0, reactivity_1.computed)(() => {
                if (plugin.getEmbeddedFileNames) {
                    const embeddedFileNames = plugin.getEmbeddedFileNames(this.fileName, this.sfc);
                    for (const oldFileName of Object.keys(embeddedFiles)) {
                        if (!embeddedFileNames.includes(oldFileName)) {
                            delete embeddedFiles[oldFileName];
                        }
                    }
                    for (const embeddedFileName of embeddedFileNames) {
                        if (!embeddedFiles[embeddedFileName]) {
                            embeddedFiles[embeddedFileName] = (0, reactivity_1.computed)(() => {
                                const file = {
                                    fileName: embeddedFileName,
                                    capabilities: {
                                        diagnostics: false,
                                        foldingRanges: false,
                                        formatting: false,
                                        documentSymbol: false,
                                        codeActions: false,
                                        inlayHints: false,
                                    },
                                    isTsHostFile: false,
                                    codeGen: new code_gen_1.CodeGen(),
                                    teleportMappings: [],
                                };
                                for (const plugin of this.plugins) {
                                    if (plugin.resolveEmbeddedFile) {
                                        plugin.resolveEmbeddedFile(this.fileName, this.sfc, file);
                                    }
                                }
                                return file;
                            });
                        }
                    }
                }
                return Object.values(embeddedFiles);
            });
            return (0, reactivity_1.computed)(() => {
                const self = this;
                const baseOffsetMap = new Map();
                return files.value.map(_file => {
                    const file = _file.value;
                    const node = {
                        fileName: file.fileName,
                        text: file.codeGen.getText(),
                        capabilities: file.capabilities,
                        isTsHostFile: file.isTsHostFile,
                        mappings: file.codeGen.mappings.map(mapping => {
                            return Object.assign(Object.assign({}, mapping), { data: mapping.data.capabilities, sourceRange: embeddedRangeToVueRange(mapping.data, mapping.sourceRange), additional: mapping.additional ? mapping.additional.map(add => {
                                    const addVueRange = embeddedRangeToVueRange(mapping.data, add.sourceRange);
                                    return Object.assign(Object.assign({}, add), { sourceRange: addVueRange });
                                }) : undefined });
                        }),
                        teleportMappings: file.teleportMappings,
                        embeddeds: [],
                    };
                    return [file, node];
                });
                function embeddedRangeToVueRange(data, range) {
                    var _a;
                    if (data.vueTag) {
                        const key = data.vueTag + '-' + data.vueTagIndex;
                        let baseOffset = baseOffsetMap.get(key);
                        if (baseOffset === undefined) {
                            if (data.vueTag === 'script' && self.sfc.script) {
                                baseOffset = self.sfc.script.startTagEnd;
                            }
                            else if (data.vueTag === 'scriptSetup' && self.sfc.scriptSetup) {
                                baseOffset = self.sfc.scriptSetup.startTagEnd;
                            }
                            else if (data.vueTag === 'template' && self.sfc.template) {
                                baseOffset = self.sfc.template.startTagEnd;
                            }
                            else if (data.vueTag === 'style') {
                                baseOffset = self.sfc.styles[data.vueTagIndex].startTagEnd;
                            }
                            else if (data.vueTag === 'customBlock') {
                                baseOffset = self.sfc.customBlocks[data.vueTagIndex].startTagEnd;
                            }
                            if (baseOffset !== undefined) {
                                baseOffsetMap.set(key, baseOffset);
                            }
                        }
                        if (baseOffset !== undefined) {
                            return {
                                start: baseOffset + range.start,
                                end: baseOffset + range.end,
                            };
                        }
                    }
                    if (data.vueTag === 'scriptSrc' && ((_a = self.sfc.script) === null || _a === void 0 ? void 0 : _a.src)) {
                        const vueStart = self._text.value.substring(0, self.sfc.script.startTagEnd).lastIndexOf(self.sfc.script.src);
                        const vueEnd = vueStart + self.sfc.script.src.length;
                        return {
                            start: vueStart - 1,
                            end: vueEnd + 1,
                        };
                    }
                    return range;
                }
            });
        });
        this._allEmbeddeds = (0, reactivity_1.computed)(() => {
            const all = [];
            for (const embeddedFiles of this._pluginEmbeddedFiles) {
                for (const embedded of embeddedFiles.value) {
                    all.push(embedded);
                }
            }
            return all;
        });
        this._embeddeds = (0, reactivity_1.computed)(() => {
            const childs = [];
            // const embeddeds: EmbeddedStructure[] = [];
            let remain = [...this._allEmbeddeds.value];
            while (remain.length) {
                const beforeLength = remain.length;
                consumeRemain();
                if (beforeLength === remain.length) {
                    break;
                }
            }
            for (const [embedded, node] of remain) {
                childs.push(node);
                if (embedded) {
                    console.error('Unable to resolve embedded: ' + embedded.parentFileName + ' -> ' + embedded.fileName);
                }
            }
            return childs;
            function consumeRemain() {
                for (let i = remain.length - 1; i >= 0; i--) {
                    const [embedded, node] = remain[i];
                    if (!embedded.parentFileName) {
                        childs.push(node);
                        remain.splice(i, 1);
                    }
                    else {
                        const parent = findParentStructure(embedded.parentFileName, childs);
                        if (parent) {
                            parent.embeddeds.push(node);
                            remain.splice(i, 1);
                        }
                    }
                }
            }
            function findParentStructure(fileName, strus) {
                for (const stru of strus) {
                    if (stru.fileName === fileName) {
                        return stru;
                    }
                    let _stru = findParentStructure(fileName, stru.embeddeds);
                    if (_stru) {
                        return _stru;
                    }
                }
            }
        });
        this._snapshot = (0, reactivity_1.shallowRef)(this.pscriptSnapshot);
        this.update(this._snapshot.value, true);
    }
    get text() {
        return this._text.value;
    }
    get compiledSFCTemplate() {
        return this._compiledSFCTemplate.value;
    }
    get tsFileName() {
        var _a, _b;
        return (_b = (_a = this._allEmbeddeds.value.find(e => e[1].fileName.replace(this.fileName, '').match(/^\.(js|ts)x?$/))) === null || _a === void 0 ? void 0 : _a[1].fileName) !== null && _b !== void 0 ? _b : '';
    }
    get embeddeds() {
        return this._embeddeds.value;
    }
    update(newScriptSnapshot, init = false) {
        const self = this;
        if (newScriptSnapshot === this._snapshot.value && !init) {
            return;
        }
        this._snapshot.value = newScriptSnapshot;
        // TODO: wait for https://github.com/vuejs/core/pull/5912
        if (this._parsedSfc.value) {
            updateTemplate(this._parsedSfc.value.descriptor.template);
            updateScript(this._parsedSfc.value.descriptor.script);
            updateScriptSetup(this._parsedSfc.value.descriptor.scriptSetup);
            updateStyles(this._parsedSfc.value.descriptor.styles);
            updateCustomBlocks(this._parsedSfc.value.descriptor.customBlocks);
        }
        else {
            updateTemplate(null);
            updateScript(null);
            updateScriptSetup(null);
            updateStyles([]);
            updateCustomBlocks([]);
        }
        function updateTemplate(block) {
            var _a;
            const newData = block ? {
                tag: 'template',
                start: self._text.value.substring(0, block.loc.start.offset).lastIndexOf('<'),
                end: block.loc.end.offset + self._text.value.substring(block.loc.end.offset).indexOf('>') + 1,
                startTagEnd: block.loc.start.offset,
                endTagStart: block.loc.end.offset,
                content: block.content,
                lang: (_a = block.lang) !== null && _a !== void 0 ? _a : 'html',
            } : null;
            if (self.sfc.template && newData) {
                updateBlock(self.sfc.template, newData);
            }
            else {
                self.sfc.template = newData;
            }
        }
        function updateScript(block) {
            var _a;
            const newData = block ? {
                tag: 'script',
                start: self._text.value.substring(0, block.loc.start.offset).lastIndexOf('<'),
                end: block.loc.end.offset + self._text.value.substring(block.loc.end.offset).indexOf('>') + 1,
                startTagEnd: block.loc.start.offset,
                endTagStart: block.loc.end.offset,
                content: block.content,
                lang: (_a = block.lang) !== null && _a !== void 0 ? _a : 'js',
                src: block.src,
            } : null;
            if (self.sfc.script && newData) {
                updateBlock(self.sfc.script, newData);
            }
            else {
                self.sfc.script = newData;
            }
        }
        function updateScriptSetup(block) {
            var _a;
            const newData = block ? {
                tag: 'scriptSetup',
                start: self._text.value.substring(0, block.loc.start.offset).lastIndexOf('<'),
                end: block.loc.end.offset + self._text.value.substring(block.loc.end.offset).indexOf('>') + 1,
                startTagEnd: block.loc.start.offset,
                endTagStart: block.loc.end.offset,
                content: block.content,
                lang: (_a = block.lang) !== null && _a !== void 0 ? _a : 'js',
            } : null;
            if (self.sfc.scriptSetup && newData) {
                updateBlock(self.sfc.scriptSetup, newData);
            }
            else {
                self.sfc.scriptSetup = newData;
            }
        }
        function updateStyles(blocks) {
            var _a;
            for (let i = 0; i < blocks.length; i++) {
                const block = blocks[i];
                const newData = {
                    tag: 'style',
                    start: self._text.value.substring(0, block.loc.start.offset).lastIndexOf('<'),
                    end: block.loc.end.offset + self._text.value.substring(block.loc.end.offset).indexOf('>') + 1,
                    startTagEnd: block.loc.start.offset,
                    endTagStart: block.loc.end.offset,
                    content: block.content,
                    lang: (_a = block.lang) !== null && _a !== void 0 ? _a : 'css',
                    module: typeof block.module === 'string' ? block.module : block.module ? '$style' : undefined,
                    scoped: !!block.scoped,
                };
                if (self.sfc.styles.length > i) {
                    updateBlock(self.sfc.styles[i], newData);
                }
                else {
                    self.sfc.styles.push(newData);
                }
            }
            while (self.sfc.styles.length > blocks.length) {
                self.sfc.styles.pop();
            }
        }
        function updateCustomBlocks(blocks) {
            var _a;
            for (let i = 0; i < blocks.length; i++) {
                const block = blocks[i];
                const newData = {
                    tag: 'customBlock',
                    start: self._text.value.substring(0, block.loc.start.offset).lastIndexOf('<'),
                    end: block.loc.end.offset + self._text.value.substring(block.loc.end.offset).indexOf('>') + 1,
                    startTagEnd: block.loc.start.offset,
                    endTagStart: block.loc.end.offset,
                    content: block.content,
                    lang: (_a = block.lang) !== null && _a !== void 0 ? _a : 'txt',
                    type: block.type,
                };
                if (self.sfc.customBlocks.length > i) {
                    updateBlock(self.sfc.customBlocks[i], newData);
                }
                else {
                    self.sfc.customBlocks.push(newData);
                }
            }
            while (self.sfc.customBlocks.length > blocks.length) {
                self.sfc.customBlocks.pop();
            }
        }
        function updateBlock(oldBlock, newBlock) {
            for (let key in newBlock) {
                oldBlock[key] = newBlock[key];
            }
        }
    }
}
exports.VueSourceFile = VueSourceFile;
//# sourceMappingURL=sourceFile.js.map