Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveVueCompilerOptions = exports.createParsedCommandLine = exports.createParsedCommandLineByJson = void 0;
const path = require("path");
function createParsedCommandLineByJson(ts, parseConfigHost, rootDir, json) {
    const tsConfigPath = path.join(rootDir, 'jsconfig.json');
    const content = ts.parseJsonConfigFileContent(json, parseConfigHost, rootDir, {}, tsConfigPath);
    return createParsedCommandLineBase(ts, parseConfigHost, content, tsConfigPath, new Set());
}
exports.createParsedCommandLineByJson = createParsedCommandLineByJson;
function createParsedCommandLine(ts, parseConfigHost, tsConfigPath, extendsSet = new Set()) {
    const config = ts.readJsonConfigFile(tsConfigPath, parseConfigHost.readFile);
    const content = ts.parseJsonSourceFileConfigFileContent(config, parseConfigHost, path.dirname(tsConfigPath), {}, path.basename(tsConfigPath));
    // fix https://github.com/johnsoncodehk/volar/issues/1786
    // https://github.com/microsoft/TypeScript/issues/30457
    // patching ts server broke with outDir + rootDir + composite/incremental
    content.options.outDir = undefined;
    return createParsedCommandLineBase(ts, parseConfigHost, content, tsConfigPath, extendsSet);
}
exports.createParsedCommandLine = createParsedCommandLine;
function createParsedCommandLineBase(ts, parseConfigHost, content, tsConfigPath, extendsSet) {
    var _a;
    let baseVueOptions = {};
    const folder = path.dirname(tsConfigPath);
    extendsSet.add(tsConfigPath);
    if (content.raw.extends) {
        try {
            const extendsPath = require.resolve(content.raw.extends, { paths: [folder] });
            if (!extendsSet.has(extendsPath)) {
                baseVueOptions = createParsedCommandLine(ts, parseConfigHost, extendsPath, extendsSet).vueOptions;
            }
        }
        catch (error) {
            console.error(error);
        }
    }
    return Object.assign(Object.assign({}, content), { vueOptions: Object.assign(Object.assign({}, baseVueOptions), resolveVueCompilerOptionsWorker((_a = content.raw.vueCompilerOptions) !== null && _a !== void 0 ? _a : {}, folder)) });
}
function resolveVueCompilerOptions(vueOptions) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o;
    const target = (_a = vueOptions.target) !== null && _a !== void 0 ? _a : 3;
    return Object.assign(Object.assign({}, vueOptions), { target, jsxTemplates: (_b = vueOptions.jsxTemplates) !== null && _b !== void 0 ? _b : false, strictTemplates: (_c = vueOptions.strictTemplates) !== null && _c !== void 0 ? _c : false, plugins: (_d = vueOptions.plugins) !== null && _d !== void 0 ? _d : [], 
        // experimental
        experimentalComponentOptionsWrapper: (_e = vueOptions.experimentalComponentOptionsWrapper) !== null && _e !== void 0 ? _e : (target >= 2.7
            ? [`(await import('vue')).defineComponent(`, `)`]
            : [`(await import('vue')).default.extend(`, `)`]), experimentalComponentOptionsWrapperEnable: (_f = vueOptions.experimentalComponentOptionsWrapperEnable) !== null && _f !== void 0 ? _f : 'onlyJs', experimentalRuntimeMode: (_g = vueOptions.experimentalRuntimeMode) !== null && _g !== void 0 ? _g : 'runtime-dom', experimentalDowngradePropsAndEmitsToSetupReturnOnScriptSetup: (_h = vueOptions.experimentalDowngradePropsAndEmitsToSetupReturnOnScriptSetup) !== null && _h !== void 0 ? _h : 'onlyJs', experimentalTemplateCompilerOptions: (_j = vueOptions.experimentalTemplateCompilerOptions) !== null && _j !== void 0 ? _j : {}, experimentalTemplateCompilerOptionsRequirePath: (_k = vueOptions.experimentalTemplateCompilerOptionsRequirePath) !== null && _k !== void 0 ? _k : undefined, experimentalDisableTemplateSupport: (_l = vueOptions.experimentalDisableTemplateSupport) !== null && _l !== void 0 ? _l : false, experimentalResolveStyleCssClasses: (_m = vueOptions.experimentalResolveStyleCssClasses) !== null && _m !== void 0 ? _m : 'scoped', experimentalAllowTypeNarrowingInInlineHandlers: (_o = vueOptions.experimentalAllowTypeNarrowingInInlineHandlers) !== null && _o !== void 0 ? _o : false });
}
exports.resolveVueCompilerOptions = resolveVueCompilerOptions;
function resolveVueCompilerOptionsWorker(rawOptions, rootPath) {
    const result = Object.assign({}, rawOptions);
    let templateOptionsPath = rawOptions.experimentalTemplateCompilerOptionsRequirePath;
    if (templateOptionsPath) {
        if (!path.isAbsolute(templateOptionsPath)) {
            templateOptionsPath = require.resolve(templateOptionsPath, { paths: [rootPath] });
        }
        try {
            result.experimentalTemplateCompilerOptions = require(templateOptionsPath).default;
        }
        catch (error) {
            console.warn('Failed to require "experimentalTemplateCompilerOptionsRequirePath":', templateOptionsPath);
            console.error(error);
        }
    }
    return result;
}
//# sourceMappingURL=ts.js.map